S is a super simple framework for building serverless libraries on AWS Lambda. It is a command line tool for your backend, and a library for your frontend.

# Table of Contents
1. [Quick Start](#quick-start)
2. [Configuration](#configuration)
    - [Regions](#regions)
    - [Stages](#stages)
    - [Domains](#domains)
      - [Host Domain](#host-domain)
      - [Origin Domains](#origin-domains)

# Quick Start

Install S:

```
npm i -g @eahefnawy/s
```

Create an **index.js** file, and start building your serverless library:

```js
const createUser = (args) => {
  // create user here...
  return user
}

const updateUser = (args) => {
  // update user here...
  return user
}

const deleteUser = (args) => {
  // delete user here...
  return user
}

module.exports = { createUser, updateUser, deleteUser }
```

Sync your serverless library. Notice the returned **SID**:

```
$ s
✔ SID: qwertyuiop (us-east-1)
$
```

Now you can use your new serverless library by passing the **SID**:

```js
const S = require('@eahefnawy/s')

const { createUser, updateUser, deleteUser } = await S('qwertyuiop') // your SID

const createdUser = await createUser(args)

const updatedUser = await updateUser(args) 

const deletedUser = await deleteUser(args)
```

Check the logs of your serverless library:

```
$ s logs
Today at 12:02:14 AM Function Called: createUser
Today at 12:02:14 AM Created User eahefnawy
Today at 12:25:59 AM Function Called: createUser
Today at 12:25:59 AM Created User eahefnawy
```

# Configuration
You can configure your library by exporting a **config** object in **index.js**:

```js
const createUser = async (args) => {
  // create user here...
  return user
}

const updateUser = async (args) => {
  // update user here...
  return user
}

const deleteUser = async (args) => {
  // delete user here...
  return user
}

const config = {
  name: 'users', // defaults to parent dir name
  region: 'us-east-2', // defaults to us-east-1
  description: 'library of user functions', // defaults to "Serverless, Simplified"
  memory: 512, // defaults to 128
  timeout: 300, // defaults to 10
  env: {
    SOME_TOKEN: 'abc'
  },
  origins: ['example.com'] // defaults to *
}

module.exports = { createUser, updateUser, deleteUser, config }
```

Some of these configuration might need to be reflected in the frontend as well when you load your library. Read more below.

## Regions
The default region of your library is `us-east-1`. You can overwrite this with the config object in **index.js**:

```js

module.exports.config = {
  region: 'us-east-2'
}
```

After running `s`, your library will be synced to that region and return a new **SID**, you can then use this region in your frontend:

```js
const S = require('@eahefnawy/s')

const config = {
  SID: 'qwertyuiop',
  region: 'us-east-2'
}

const { createUser, updateUser, deleteUser } = await S(config)
```

## Stages
To have multiple stages of your library, you simply change the name of the library in the config object of **index.js**:

```js
module.exports.config = {
  name: 'dev'
}
```

Then sync your library again, this will sync a new version of your library and return a new **SID** for you to use in your frontend. Once you've tested everything and ready to go to the next stage, simply change the name to `beta` for example.

When working within a team, you can all collaborate on the same stage by using the same library name, in that case your work will overwrite each other, or each team member could have their own stage, and once they finish testing without interfering with other, they can sync to a shared stage.

**Pro Tip:** Make use of environment variables to manage all these stages, and keep the code "stage-agnostic".

## Domains
During development, you can use your **SID** to load your library, but remember that this might expose your **SID** and anyone in the world who has this **SID** can load your library. Once you're ready for production, we recommend using custom domains instead of **SID** to load and secure your library.

### Host Domain
If you have aliased your AWS API Gateway endpoint with a a domain or subdomain, you can use that as the host domain instead of exposing your **SID**:

```js
const S = require('@eahefnawy/s')

const { createUser, updateUser, deleteUser } = await S('https://api.example.com') // Your host domain
```

Setting up this domain is beyond the scope of this project. But you can easily do that with AWS CloudFront.

### Origin Domains

Configuring host domain doesn't help secure your library to be used only by your application. To do that. you will need to configure your library to only accept requests from certain domains (also known as CORS). You can do that with the config object:


```js
module.exports.config = {
  origins: ['example.com']
}
```
This will only allow your library to be loaded from `example.com`, so that no one else can use your library.
